@@ -1,4 +1,4 @@
-/*	$Id: kcgi.h,v 1.130 2019/08/10 08:17:34 kristaps Exp $ */
+/*	$Id: kcgi.h,v 1.151 2020/04/10 12:58:29 kristaps Exp $ */
 /*
  * Copyright (c) 2012, 2014--2018 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -48,12 +48,12 @@
 /*
  * Minor version.
  */
-#define	KCGI_VMINOR	10
+#define	KCGI_VMINOR	12
 
 /*
  * Build version.
  */
-#define	KCGI_VBUILD	11
+#define	KCGI_VBUILD	0
 
 /*
  * Version string of major.minor.build (as a literal string).
@@ -499,7 +499,9 @@ enum	kcgi_err {
 	/* Internal system error (malformed data). */
 	KCGI_FORM,
 	/* Opaque operating-system error. */
-	KCGI_SYSTEM
+	KCGI_SYSTEM,
+	/* Writer error. */
+	KCGI_WRITER
 };
 
 typedef enum kcgi_err (*ktemplate_writef)(const char *, size_t, void *);
@@ -529,6 +531,8 @@ enum kcgi_err	 khttp_parsex(struct kreq *, const struct kmimemap *,
 			const char *const *, size_t,
 			size_t, size_t, void *, void (*)(void *),
 			unsigned int, const struct kopts *);
+enum kcgi_err	 khttp_printf(struct kreq *, const char *, ...)
+			__attribute__((format(printf, 2, 3)));
 enum kcgi_err	 khttp_putc(struct kreq *, int);
 enum kcgi_err	 khttp_puts(struct kreq *, const char *);
 enum kcgi_err	 khttp_template(struct kreq *, 
@@ -547,12 +551,13 @@ enum kcgi_err	 khttp_templatex_buf(const struct ktemplate *,
 enum kcgi_err	 khttp_templatex_fd(const struct ktemplate *, 
 			int, const char *,
 			const struct ktemplatex *, void *);
-
 enum kcgi_err	 khttp_write(struct kreq *, const char *, size_t);
 
-enum kcgi_err	 kcgi_buf_write(const char *, size_t, void *);
+enum kcgi_err	 kcgi_buf_printf(struct kcgi_buf *, const char *, ...)
+			__attribute__((format(printf, 2, 3)));
 enum kcgi_err	 kcgi_buf_putc(struct kcgi_buf *, char);
 enum kcgi_err	 kcgi_buf_puts(struct kcgi_buf *, const char *);
+enum kcgi_err	 kcgi_buf_write(const char *, size_t, void *);
 
 int		 khttpdigest_validate(const struct kreq *, 
 			const char *);
@@ -609,34 +614,64 @@ int64_t	 	 kutil_datetime2epoch(int64_t, int64_t, int64_t,
 int	 	 kutil_datetime_check(int64_t, int64_t, int64_t,
 			int64_t, int64_t, int64_t);
 
+char 		*khttp_urlabs(enum kscheme, const char *, 
+			uint16_t, const char *, ...);
+enum kcgi_err	 khttp_urldecode(const char *, char **);
+enum kcgi_err	 khttp_urldecode_inplace(char *);
+char		*khttp_urlencode(const char *);
+char		*khttp_urlpart(const char *,
+			const char *, const char *, ...);
+char		*khttp_urlpartx(const char *,
+			const char *, const char *, ...);
+char 		*khttp_vurlabs(enum kscheme, const char *, 
+			uint16_t, const char *, va_list);
+char		*khttp_vurlpart(const char *,
+			const char *, const char *, va_list);
+char		*khttp_vurlpartx(const char *,
+			const char *, const char *, va_list);
+
 char		*kutil_urlabs(enum kscheme, const char *, 
-			uint16_t, const char *);
+			uint16_t, const char *)
+			__attribute((deprecated));
+enum kcgi_err	 kutil_urldecode(const char *, char **)
+			__attribute((deprecated));
+enum kcgi_err	 kutil_urldecode_inplace(char *)
+			__attribute((deprecated));
+char		*kutil_urlencode(const char *)
+			__attribute((deprecated));
 char		*kutil_urlpart(struct kreq *, const char *,
-			const char *, const char *, ...);
+			const char *, const char *, ...)
+			__attribute((deprecated));
 char		*kutil_urlpartx(struct kreq *, const char *,
-			const char *, const char *, ...);
-char		*kutil_urlencode(const char *);
-enum kcgi_err	 kutil_urldecode(const char *, char **);
-enum kcgi_err	 kutil_urldecode_inplace(char *);
+			const char *, const char *, ...)
+			__attribute((deprecated));
+
 void		 kutil_invalidate(struct kreq *, struct kpair *);
 
 int		 kutil_openlog(const char *);
 void	 	 kutil_verr(const struct kreq *, 
 			const char *, const char *, va_list)
+			__attribute__((format(printf, 3, 0)))
 			__attribute__((__noreturn__));
 void	 	 kutil_verrx(const struct kreq *, 
 			const char *, const char *, va_list)
+			__attribute__((format(printf, 3, 0)))
 			__attribute__((__noreturn__));
 void		 kutil_vinfo(const struct kreq *, 
-			const char *, const char *, va_list);
+			const char *, const char *, va_list)
+			__attribute__((format(printf, 3, 0)));
 void		 kutil_vlog(const struct kreq *, const char *,
-			const char *, const char *, va_list);
+			const char *, const char *, va_list)
+			__attribute__((format(printf, 4, 0)));
 void		 kutil_vlogx(const struct kreq *, const char *,
-			const char *, const char *, va_list);
+			const char *, const char *, va_list)
+			__attribute__((format(printf, 4, 0)));
 void		 kutil_vwarn(const struct kreq *, 
-			const char *, const char *, va_list);
+			const char *, const char *, va_list)
+			__attribute__((format(printf, 3, 0)));
 void		 kutil_vwarnx(const struct kreq *, 
-			const char *, const char *, va_list);
+			const char *, const char *, va_list)
+			__attribute__((format(printf, 3, 0)));
 void		 kutil_log(const struct kreq *, const char *,
 			const char *, const char *, ...)
 			__attribute__((format(printf, 4, 5)));
@@ -663,6 +698,8 @@ void		 kutil_errx(const struct kreq *,
 
 int		 kasprintf(char **, const char *, ...)
 			__attribute__((format(printf, 2, 3)));
+int		 kvasprintf(char **, const char *, va_list)
+			__attribute__((format(printf, 2, 0)));
 void		*kcalloc(size_t, size_t);
 void		*kmalloc(size_t);
 void		*krealloc(void *, size_t);
