@@ -1,4 +1,4 @@
-/*	$Id: kcgihtml.h,v 1.10 2018/01/12 14:36:22 kristaps Exp $ */
+/*	$Id: kcgihtml.h,v 1.15 2020/03/27 22:14:15 kristaps Exp $ */
 /*
  * Copyright (c) 2012, 2014, 2015, 2017 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -452,10 +452,15 @@ enum	kelem {
 	KELEM__MAX
 };
 
+/*
+ * The maximum number of HTML scopes allowed.
+ * Once this has been reached, new scopes will return KCGI_ENOMEM.
+ */
+#define	KHTML_STACK_MAX	128
+
 struct	khtmlreq {
 	void		*arg;
-#define	KDATA_MAXELEMSZ	 128
-	enum kelem	 elems[KDATA_MAXELEMSZ];
+	enum kelem	 elems[KHTML_STACK_MAX];
 	size_t		 elemsz;
 	int		 newln;
 	int		 opts;
@@ -471,12 +476,14 @@ enum kcgi_err	 khtml_closeelem(struct khtmlreq *, size_t);
 enum kcgi_err	 khtml_closeto(struct khtmlreq *, size_t);
 enum kcgi_err	 khtml_double(struct khtmlreq *, double);
 enum kcgi_err	 khtml_elem(struct khtmlreq *, enum kelem);
-size_t		 khtml_elemat(struct khtmlreq *);
+size_t		 khtml_elemat(const struct khtmlreq *);
 enum kcgi_err	 khtml_entity(struct khtmlreq *, enum kentity);
 enum kcgi_err	 khtml_int(struct khtmlreq *, int64_t);
-enum kcgi_err	 khtml_ncr(struct khtmlreq *, uint16_t);
+enum kcgi_err	 khtml_ncr(struct khtmlreq *, uint32_t);
 enum kcgi_err 	 khtml_open(struct khtmlreq *, struct kreq *, int);
 enum kcgi_err	 khtml_putc(struct khtmlreq *, char);
+enum kcgi_err	 khtml_printf(struct khtmlreq *, const char *, ...)
+			__attribute__((format(printf, 2, 3)));
 enum kcgi_err	 khtml_puts(struct khtmlreq *, const char *);
 enum kcgi_err	 khtml_write(const char *, size_t, void *);
 
